/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.domain;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Anfrage an den Ereigniskalender zur Bestimmung von Ereigniswechseln in einem
 * vorgegebenen Zeitraum.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class EreignisKalenderAnfrage {

	/** Optionen für Typ-Filter. */
	public enum EreignisTypenOption {
		ALLE(0), NUR(1), NICHT(2);

		private final int davWert;

		EreignisTypenOption(int davWert) {
			this.davWert = davWert;
		}

		public int getDavWert() {
			return davWert;
		}
	}

	private Intervall intervall;
	private final List<String> rauemlicheGueltigkeit = new ArrayList<>();
	private final String absenderZeichen;
	private final List<EreignisTyp> ereignisTypen = new ArrayList<>();
	private EreignisTypenOption ereignisTypenOption = EreignisTypenOption.ALLE;

	public EreignisKalenderAnfrage(String absenderZeichen) {
		this.absenderZeichen = absenderZeichen;

	}

	public void setIntervall(Intervall intervall) {
		this.intervall = intervall;
	}

	public Intervall getIntervall() {
		return intervall;
	}

	public void addRaeumlicheGueltigkeit(String pid) {
		if (pid != null) {
			rauemlicheGueltigkeit.add(pid);
		}
	}

	public List<String> getRaeumlicheGueltigkeit() {
		return Collections.unmodifiableList(rauemlicheGueltigkeit);
	}

	public String getAbsenderZeichen() {
		return absenderZeichen;
	}

	public EreignisTypenOption getEreignisTypenOption() {
		return ereignisTypenOption;
	}

	public void addEreignisTyp(EreignisTyp ereignisTyp) {
		ereignisTypen.add(ereignisTyp);
	}

	public List<EreignisTyp> getEreignisTypen() {
		return Collections.unmodifiableList(ereignisTypen);
	}
}
