/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.domain.comm;

import java.util.Collections;
import java.util.List;

import de.bsvrz.iav.gllib.gllib.domain.GanglinieAnfrageTyp;
import de.bsvrz.iav.gllib.gllib.domain.GanglinienAnfrager;
import de.bsvrz.iav.gllib.gllib.domain.MessQuerschnitt;
import de.bsvrz.iav.gllib.gllib.domain.MqGanglinie;

/**
 * Anfrage zum Schreiben von historischen Ganglinien im Ganglinienspeicher.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public final class GlSchreibenAnfrage extends GlSpeicherAnfrage {

    private GlSchreibenAnfrage(GanglinienAnfrager anfrager, String absenderZeichen, MessQuerschnitt messQuerschnitt,
            List<MqGanglinie> ganglinien) {
        super(GanglinieAnfrageTyp.SCHREIBEN, anfrager, absenderZeichen, messQuerschnitt, Collections.emptyList(), Collections.emptyList(), ganglinien);
    }

    public static GlSchreibenAnfrage of(GanglinienAnfrager anfrager, String absenderZeichen,
            List<MqGanglinie> ganglinien) {
        
        MessQuerschnitt messQuerschnitt = null;
        for (MqGanglinie ganglinie : ganglinien) {
            MessQuerschnitt ganglinieMq = ganglinie.getMessQuerschnitt();
            if (ganglinieMq == null) {
                throw new IllegalArgumentException("Eine zu schreibende Ganglinie muss einen zugeordneten MQ haben");
            }
            if (messQuerschnitt == null || ganglinieMq.equals(messQuerschnitt)) {
                messQuerschnitt = ganglinieMq;
            } else {
                throw new IllegalArgumentException(
                        "Die Liste der zu schreibenden Ganglinien muss dem gleichen Messquerschnitt zugeordnet sein");
            }
        }

        return new GlSchreibenAnfrage(anfrager, absenderZeichen, messQuerschnitt, ganglinien);
    }
}
