/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.portals;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.iav.gllib.gllib.domain.Ereignis;
import de.bsvrz.iav.gllib.gllib.shared.Event;

/**
 * Beobachter für die Menge der Ereignisse innerhal einer AOE.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavEreignisMengenObserver {

    private final Map<String, Ereignis> ereignisse = new LinkedHashMap<>();
    private final Event<Boolean> mengenAenderung = new Event<>();
    private final DavObjektFactory objektFactory;

    public DavEreignisMengenObserver(DavObjektFactory objektFactory) {

        this.objektFactory = objektFactory;

        ClientDavInterface dav = objektFactory.getDav();

        MutableSet ereignisSet = (MutableSet) dav.getLocalConfigurationAuthority().getObjectSet("Ereignisse");
        for (SystemObject so : ereignisSet.getElements()) {
            ereignisse.put(so.getPid(), objektFactory.createEreignis(so.getPid()));
        }

        ereignisSet.addChangeListener((set, added, removed) -> setChanged(set, added, removed));
    }

    private void setChanged(@SuppressWarnings("unused") MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects) {
        for (SystemObject so : removedObjects) {
            ereignisse.remove(so.getPid());
        }

        for (SystemObject so : addedObjects) {
            ereignisse.put(so.getPid(), objektFactory.createEreignis(so.getPid()));
        }

        mengenAenderung.send(true);
    }

    public Set<Ereignis> getEreignisse() {
        return new LinkedHashSet<>(ereignisse.values());
    }

    public Event<Boolean> mengenAenderung() {
        return mengenAenderung;
    }
}
