/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.domain.comm;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;

import de.bsvrz.iav.gllib.gllib.domain.EreignisTyp;
import de.bsvrz.iav.gllib.gllib.domain.GanglinieAnfrageTyp;
import de.bsvrz.iav.gllib.gllib.domain.GanglinienAnfrager;
import de.bsvrz.iav.gllib.gllib.domain.MessQuerschnitt;
import de.bsvrz.iav.gllib.gllib.domain.MqGanglinie;

/**
 * Anfrage zum Löschen von historischen Ganglinien vom Ganglinienspeicher.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public final class GlLoeschenAnfrage extends GlSpeicherAnfrage {

    private GlLoeschenAnfrage(GanglinienAnfrager anfrager, String absenderZeichen, MessQuerschnitt messQuerschnitt,
            Collection<EreignisTyp> ereignisTypen, Collection<Long> ganglinienId) {
        super(GanglinieAnfrageTyp.LOESCHEN, anfrager, absenderZeichen, messQuerschnitt, ereignisTypen, ganglinienId, Collections.emptyList());
    }

    public static GlLoeschenAnfrage of(GanglinienAnfrager anfrager, String absenderZeichen,
            List<MqGanglinie> ganglinien) {
        if (ganglinien.isEmpty()) {
            throw new IllegalArgumentException("Die Liste der zu löschenden Ganglinien darf nicht leer sein");
        }

        MessQuerschnitt messQuerschnitt = null;
        Collection<Long> ganglinienId = new LinkedHashSet<>();
        for (MqGanglinie ganglinie : ganglinien) {
            MessQuerschnitt ganglinieMq = ganglinie.getMessQuerschnitt();
            if (ganglinieMq == null) {
                throw new IllegalArgumentException("Eine zu löschende Ganglinie muss einen zugeordneten MQ haben");
            }
            if (messQuerschnitt == null || ganglinieMq.equals(messQuerschnitt)) {
                messQuerschnitt = ganglinieMq;
                ganglinienId.add(ganglinie.getGanglinieId());
            } else {
                throw new IllegalArgumentException(
                        "Die Liste der zu löschenden Ganglinien muss dem gleichen Messquerschnitt zugeordnet sein");
            }
        }

        return new GlLoeschenAnfrage(anfrager, absenderZeichen, messQuerschnitt, Collections.emptyList(), ganglinienId);
    }

    public static GlLoeschenAnfrage of(GanglinienAnfrager anfrager, String absenderZeichen,
            MessQuerschnitt messQuerschnitt) {
        return new GlLoeschenAnfrage(anfrager, absenderZeichen, messQuerschnitt, Collections.emptyList(),
                Collections.emptyList());
    }

    public static GlLoeschenAnfrage of(GanglinienAnfrager anfrager, String absenderZeichen,
            MessQuerschnitt messQuerschnitt, Collection<EreignisTyp> ereignistypen) {
        return new GlLoeschenAnfrage(anfrager, absenderZeichen, messQuerschnitt, ereignistypen,
                Collections.emptyList());
    }

    public static GlLoeschenAnfrage of(GanglinienAnfrager anfrager, String absenderZeichen,
            MessQuerschnitt messQuerschnitt, Collection<EreignisTyp> ereignistypen, Collection<Long> ganglinienId) {
        return new GlLoeschenAnfrage(anfrager, absenderZeichen, messQuerschnitt, ereignistypen, ganglinienId);
    }
}
