/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.portals;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.iav.gllib.gllib.domain.Ereignis;
import de.bsvrz.iav.gllib.gllib.domain.EreignisKalenderAntwort;
import de.bsvrz.iav.gllib.gllib.domain.ZustandsWechsel;
import de.bsvrz.iav.gllib.gllib.shared.Event;

/**
 * 
 * Empfänger zum Empfang von Antworten auf Anfragen an den Ereigniskalender über
 * die Datenverteilerschnittstelle.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavEreignisKalenderAntwortEmpfaenger {

    private final ClientReceiverInterface receiver = new ClientReceiverInterface() {

        private Executor executor = Executors.newSingleThreadExecutor();

        @Override
        public void update(ResultData[] results) {
            for (ResultData result : results) {
                if (result.hasData()) {
                    executor.execute(() -> empfangenEvent.send(konvertiere(result.getData())));
                }
            }
        }
    };

    private final Event<EreignisKalenderAntwort> empfangenEvent = new Event<>();

    private DavObjektFactory factory;

    public DavEreignisKalenderAntwortEmpfaenger(DavObjektFactory factory) {

        this.factory = factory;

        ClientDavInterface dav = factory.getDav();
        AttributeGroup atg = dav.getDataModel().getAttributeGroup("atg.ereignisKalenderAntwort");
        Aspect asp = dav.getDataModel().getAspect("asp.antwort");
        dav.subscribeReceiver(receiver, dav.getLocalApplicationObject(), new DataDescription(atg, asp),
                ReceiveOptions.normal(), ReceiverRole.drain());
    }

    private EreignisKalenderAntwort konvertiere(Data daten) {

        String absenderzeichen = daten.getTextValue("absenderZeichen").getText();
        List<ZustandsWechsel> zustandsWechsel = new ArrayList<>();

        Array ereignisArray = daten.getArray("Ereignis");
        for (int i = 0; i < ereignisArray.getLength(); ++i) {

            Data wechselItem = ereignisArray.getItem(i);

            long zeitpunkt = wechselItem.getTimeValue("Zeitpunkt").getMillis();
            Ereignis ereignis = factory
                    .createEreignis(wechselItem.getReferenceValue("EreignisReferenz").getSystemObjectPid());
            boolean verkehrlichGueltig = "ja"
                    .equalsIgnoreCase(wechselItem.getUnscaledValue("verkehrlichGültig").getState().getName());
            boolean zeitlichGueltig = "ja"
                    .equalsIgnoreCase(wechselItem.getUnscaledValue("zeitlichGültig").getState().getName());

            zustandsWechsel.add(ZustandsWechsel.of(zeitpunkt, ereignis, verkehrlichGueltig, zeitlichGueltig));
        }

        return new EreignisKalenderAntwort(absenderzeichen, zustandsWechsel);
    }

    public Event<EreignisKalenderAntwort> antwortEmpfangen() {
        return empfangenEvent;
    }
}
